#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.


class NoUniqueMatch(RuntimeError):
    """There was more than one extension, or none, that matched the query."""


class NoMatches(NoUniqueMatch):
    """There were no extensions with the driver name found."""


class MultipleMatches(NoUniqueMatch):
    """There were multiple matches for the given name."""
